// Generated by CoffeeScript 1.12.7
(function() {
  var debug, fs, macros, minutes, pkg, seconds, seem, sleep;

  pkg = require('../../package');

  this.name = pkg.name + ":middleware:client:conference";

  debug = (require('tangible'))(this.name);

  seem = require('seem');

  fs = require('fs');

  sleep = function(timeout) {
    return new Promise(function(resolve) {
      return setTimeout(resolve, timeout);
    });
  };

  seconds = 1000;

  minutes = 60 * seconds;

  macros = function(cfg) {
    var __count, count, still_running;
    __count = function(name) {
      return cfg.api("conference " + name + " get count");
    };
    count = seem(function*(name) {
      var value;
      value = (yield __count(name));
      if (value.match(/^\d+/)) {
        return parseInt(value, 10);
      } else {
        return null;
      }
    });
    still_running = seem(function*(name) {
      return ((yield count(name))) === !null;
    });
    return {
      count: count,
      still_running: still_running
    };
  };

  this.init = function() {
    return this.cfg.statistics.on('conference:record', seem((function(_this) {
      return function*(name) {
        var last_uri, recording, still_running, uri;
        still_running = macros(_this.cfg).still_running;
        debug('conference:record', name);
        recording = (yield _this.cfg.api("conference " + name + " chkrecord"));
        debug('conference:record: recording', name, recording);
        if (!(recording != null ? recording.match(/is not being recorded/) : void 0)) {
          debug('conference:record: Already recording or not ready', name, recording);
          return;
        }
        if (_this.cfg.recording_uri == null) {
          debug.dev('conference:record: Missing recording_uri', name);
          return;
        }
        uri = (yield _this.cfg.recording_uri(name));
        yield _this.cfg.api("conference " + name + " recording start " + uri);
        yield _this.cfg.api("conference " + name + " play tone_stream://%(125,0,400);%(125,0,450);%(125,0,400)");
        last_uri = uri;
        while ((yield still_running(name))) {
          yield sleep(29 * minutes);
          uri = (yield _this.cfg.recording_uri(name));
          yield _this.cfg.api("conference " + name + " recording start " + uri);
          yield _this.cfg.api("conference " + name + " play tone_stream://%(125,0,400);%(125,0,450);%(125,0,400)");
          yield sleep(1 * minutes);
          yield _this.cfg.api("conference " + name + " recording stop " + last_uri);
          last_uri = uri;
        }
      };
    })(this)));
  };

  this.notify = function() {
    this.configure({
      dial_calls: true
    });
    this.register('conference:get-participants', 'dial_calls');
    this.register('conference:participants', 'calls');
    return this.socket.on('conference:get-participants', seem((function(_this) {
      return function*(conf_name) {
        var $, conf_data, content, domain, list;
        debug('conference:get-participants', conf_name);
        if (!($ = conf_name.match(/^(\S+)-conf-\d+$/))) {
          return;
        }
        domain = $[1];
        list = (yield _this.cfg.api("conference " + conf_name + " json_list")["catch"](function() {
          return null;
        }));
        if (list == null) {
          return;
        }
        content = (function() {
          try {
            return JSON.parse(list);
          } catch (error1) {}
        })();
        if (content == null) {
          return;
        }
        conf_data = content[0];
        if (conf_data == null) {
          return;
        }
        conf_data._in = ["number_domain:" + domain, "conference:" + conf_name];
        return _this.socket.emit('conference:participants', conf_data);
      };
    })(this)));
  };

  this.include = seem(function*() {
    var announce, authenticated, conf_name, conf_uri, count, currently, get_conf_pin, id, is_remote, language, music_uri, namefile, play_in_conference, ref, ref1, res, server, start_recording, uri;
    if (((ref = this.session) != null ? ref.direction : void 0) !== 'conf') {
      return;
    }
    if (this.cfg.host == null) {
      this.debug.dev('Missing cfg.host');
      return;
    }
    if (this.session.conf == null) {
      this.debug.dev('Missing conference data');
      return;
    }
    conf_name = this.session.conf.full_name;
    is_remote = (yield this.cfg.is_remote(conf_name, this.session.local_server));
    if (is_remote) {
      server = is_remote;
      uri = "sip:localconf-" + conf_name + "@" + server + ";xref={@session.reference}";
      this.debug('Conference is remote', uri);
      res = (yield this.action('deflect', uri));
      this.debug('Remote conference returned', uri, res);
      return;
    }
    this.debug('Conference is local');
    this.notify({
      state: 'conference',
      name: conf_name
    });
    conf_uri = (function(_this) {
      return function(id, name) {
        return _this.prompt.uri('prov', 'prov', id, name);
      };
    })(this);
    id = "number_domain:" + this.session.number_domain;
    if (this.session.music != null) {
      music_uri = this.session.music;
    }
    if (this.session.conf.music != null) {
      if (this.session.conf.music === false) {
        music_uri = null;
      } else {
        music_uri = conf_uri(id, this.session.conf.music);
      }
    }
    if (music_uri != null) {
      yield this.set({
        conference_moh_sound: music_uri
      });
    }
    language = this.session.conf.language;
    if (language == null) {
      language = this.session.language;
    }
    if (language == null) {
      language = this.cfg.announcement_language;
    }
    yield this.action('answer');
    yield this.set({
      language: language
    });
    yield this.sleep(2000);
    get_conf_pin = (function(_this) {
      return function(o) {
        if (o == null) {
          o = {};
        }
        return _this.prompt.get_pin({
          max: 8,
          tries: 3,
          timeout: 6000,
          file: 'phrase:conference:pin',
          invalid_file: 'phrase:conference:bad_pin'
        });
      };
    })(this);
    authenticated = seem((function(_this) {
      return function*() {
        var customer_pin, pin;
        pin = _this.session.conf.pin;
        _this.debug('pin', pin);
        if (pin == null) {
          return true;
        }
        customer_pin = (yield get_conf_pin()["catch"](function(error) {
          var ref1;
          _this.debug("pin error: " + ((ref1 = error.stack) != null ? ref1 : error));
          return null;
        }));
        return pin === customer_pin;
      };
    })(this));
    if ((yield authenticated())) {
      namefile = "/tmp/" + this.session.logger_uuid + "-name.wav";
      yield this.action('playback', 'phrase:voicemail_record_name');
      this.debug('record');
      yield this.action('record', namefile + " 2");
      count = macros(this.cfg).count;
      currently = (yield count(conf_name));
      if (currently == null) {
        currently = 0;
      }
      yield this.action('playback', "phrase:conference:count:" + currently);
      play_in_conference = (function(_this) {
        return function(what) {
          return _this.call.api(['conference', conf_name, 'play', what].join(' '));
        };
      })(this);
      announce = seem((function(_this) {
        return function*() {
          _this.debug('announce');
          yield play_in_conference('tone_stream://%(125,0,300);%(125,0,450);%(125,0,600)');
          return (yield play_in_conference(namefile)["catch"](function(error) {
            var ref1;
            return _this.debug("error: " + ((ref1 = error.stack) != null ? ref1 : error));
          }));
        };
      })(this));
      this.call.once('cleanup_linger', (function(_this) {
        return function*() {
          yield play_in_conference('tone_stream://%(125,0,600);%(125,0,450);%(125,0,300)');
          yield play_in_conference(namefile);
          try {
            return fs.unlinkSync(namefile);
          } catch (error1) {}
        };
      })(this));
      setTimeout(announce, 1000);
      yield this.set({
        conference_max_members: (ref1 = this.session.conf.max_members) != null ? ref1 : null
      });
      this.debug('conference');
      yield this.reference.add_in("number_domain:" + this.session.number_domain);
      this.notify({
        state: 'conference:started',
        conference: conf_name
      });
      if (this.session.conf.record) {
        start_recording = (function(_this) {
          return function() {
            return _this.cfg.statistics.emit('conference:record', conf_name);
          };
        })(this);
        setTimeout(start_recording, 1000);
      }
      yield this.action('conference', conf_name + "++flags{}");
    }
  });

}).call(this);
